function woa13_surf( time )
%
% Matlab script for creating sea surface climatology NetCDF file with data
% from WORLD OCEAN ATLAS 2013 version 2 (WOA13)
%
% Variables: temperature and salinity for 2005-2012 years with standard 
% deviations from averaged decades 1955-2012 and the number of observations
% The names of the variables show the ROSCOP codes prefixes of WOA_
% sea surface temperature -> WOA_SSTP
% sea surface salinity -> WOA_SSPS
% Data available for depth 0 and 5 m
%
%
% Data files are to be retrieved under:
% - https://www.nodc.noaa.gov/OC5/woa13/
% - http://data.nodc.noaa.gov/woa/WOA13/DATAv2/temperature/ascii/decav/1.00/
% Instructions for downloading, decompressing, and reading WOA13 data:
% - https://www.nodc.noaa.gov/OC5/woa13/readwoa13.html
%
% We use OPeNDAP NetCDF data source. 
% 
% usage:
% woa13_surf('annual')
% woa13_surf('seasonal')
% woa13_surf('monthly')
%
% $Id:$

% Create the netcdf file
filename = strcat('woa13_', time, '_surf.nc');
disp(['Create netcdf file: ' filename]);
nc = netcdf.create(filename, 'CLOBBER');

% Define useful constants:
NC_GLOBAL = netcdf.getConstant('NC_GLOBAL');
%NC_FLOAT = netcdf.getConstant('NC_FLOAT');
fillValue = single(-99.9999);

% Definitions:
% Global attributes 
netcdf.putAtt(nc, NC_GLOBAL, 'description', ['WOA13 CLIMATOLOGY Surface temperature salinity ' time]);
netcdf.putAtt(nc, NC_GLOBAL, 'author', 'J Grelet IRD US191 Brest');
netcdf.putAtt(nc, NC_GLOBAL, 'date', datestr(now, 'dd mmm yyyy'));

% Define dimensions
dimidZ = netcdf.defDim(nc, 'WOA_DEPH', 2);      % depth (surface and 5 m)
dimidX = netcdf.defDim(nc, 'WOA_LONX', 1440);   % -180, 180 1/4 degrees
dimidY = netcdf.defDim(nc, 'WOA_LATX', 720);    % -90, 90 1/4 degrees

% Define dimension T, imin and imax follow the WOA filename convention
switch time
    case 'annual'
        dimidT = netcdf.defDim(nc, 'WOA_TIME', 1);      % time
        WOA_TIME       = 1;
        imin = 1; imax = 1; offset = 0;
    case 'seasonal'
        dimidT = netcdf.defDim(nc, 'WOA_TIME', 4);
         WOA_TIME       = 1:4;
        imin = 13; imax = 16; offset = 12;
    case 'monthly'
        dimidT = netcdf.defDim(nc, 'WOA_TIME', 12);
         WOA_TIME        = 1:12;
        imin = 1; imax = 12; offset = 0;
end

% Variables
varid_time = netcdf.defVar(nc, 'WOA_TIME', 'float', dimidT);
%netcdf.defVarFill(nc, varid, false, fillValue);
varid_deph = netcdf.defVar(nc, 'WOA_DEPH', 'float', dimidZ);
netcdf.putAtt(nc, varid_deph, 'units','Depth');
netcdf.putAtt(nc, varid_deph, 'long_name','meters');
netcdf.putAtt(nc, varid_deph, '_FillValue',fillValue);
varid_lat = netcdf.defVar(nc, 'WOA_LATX', 'float', dimidY);
netcdf.putAtt(nc, varid_lat, 'units','Latitude');
netcdf.putAtt(nc, varid_lat, 'long_name','degree_north');
netcdf.putAtt(nc, varid_lat, '_FillValue',fillValue);
varid_lon = netcdf.defVar(nc, 'WOA_LONX', 'float', dimidX);
netcdf.putAtt(nc, varid_lon, 'units','Longitude');
netcdf.putAtt(nc, varid_lon, 'long_name','degree_east');
netcdf.putAtt(nc, varid_lon, '_FillValue',fillValue);

varid_mean_sstp = netcdf.defVar(nc, 'WOA_MEAN_SSTP', 'float', [dimidX,dimidY,dimidZ,dimidT]);
netcdf.putAtt(nc, varid_mean_sstp, 'units','Sea Surface Temperature mean');
netcdf.putAtt(nc, varid_mean_sstp, 'long_name','degre Celcius');
netcdf.putAtt(nc, varid_mean_sstp, '_FillValue',fillValue);
varid_mean_ssps = netcdf.defVar(nc, 'WOA_MEAN_SSPS', 'float', [dimidX,dimidY,dimidZ,dimidT]);
netcdf.putAtt(nc, varid_mean_ssps, 'units','Sea Surface Salinity mean');
netcdf.putAtt(nc, varid_mean_ssps, 'long_name','PSU');
netcdf.putAtt(nc, varid_mean_ssps, '_FillValue',fillValue);

varid_std_sstp = netcdf.defVar(nc, 'WOA_STD_SSTP', 'float', [dimidX,dimidY,dimidZ,dimidT]);
netcdf.putAtt(nc, varid_std_sstp, 'units','Sea Surface Temperature standard deviation');
netcdf.putAtt(nc, varid_std_sstp, 'long_name','degre Celcius');
netcdf.putAtt(nc, varid_std_sstp, '_FillValue',fillValue);
varid_std_ssps = netcdf.defVar(nc, 'WOA_STD_SSPS', 'float', [dimidX,dimidY,dimidZ,dimidT]);
netcdf.putAtt(nc, varid_std_ssps, 'units','Sea Surface Salinity standard deviation');
netcdf.putAtt(nc, varid_std_ssps, 'long_name','PSU');
netcdf.putAtt(nc, varid_std_ssps, '_FillValue',fillValue);

varid_obs_sstp = netcdf.defVar(nc, 'WOA_OBS_SSTP', 'short', [dimidX,dimidY,dimidZ,dimidT]);
netcdf.putAtt(nc, varid_obs_sstp, 'units','Sea Surface Temperature number observation');
netcdf.putAtt(nc, varid_obs_sstp, 'long_name','none');
netcdf.putAtt(nc, varid_obs_sstp, '_FillValue',int16(-100));
varid_obs_ssps = netcdf.defVar(nc, 'WOA_OBS_SSPS', 'short', [dimidX,dimidY,dimidZ,dimidT]);
netcdf.putAtt(nc, varid_obs_ssps, 'units','Sea Surface Salinity number observation');
netcdf.putAtt(nc, varid_obs_ssps, 'long_name','none');
netcdf.putAtt(nc, varid_obs_ssps, '_FillValue',int16(-100));

% Leave define mode.
netcdf.endDef(nc);

% Define variable(dimension)
WOA_DEPH = [5 10];
%WOA_DEPH = [0];
WOA_LATX = (-89.875:0.25:89.875)';
WOA_LONX = (-179.875:0.25:179.875)';

% write variables
netcdf.putVar(nc, varid_time, WOA_TIME);
netcdf.putVar(nc, varid_deph, WOA_DEPH);
netcdf.putVar(nc, varid_lat, WOA_LATX);
netcdf.putVar(nc, varid_lon, WOA_LONX);

% loop over WOA filename convention
for i = imin:imax
    if strcmp(time, 'annual')
        num = '00';
    else
        num = sprintf('%02d', i);
    end
    % Apparently, START must be set using the C convention, that is,
    % it seems to be zero-based!
    % thanks: https://people.sc.fsu.edu/~jburkardt/m_src/netcdf/
    start = [0,0,0,i-offset-1];
    count = [1440,720,2,1];
    
    % read and extract temperature
    [mean_t, std_t, obs_t] = rdwoa('t', num);
    % write to netcdf file
    netcdf.putVar(nc, varid_mean_sstp, start, count, mean_t);
    netcdf.putVar(nc, varid_std_sstp, start, count, std_t);
    netcdf.putVar(nc, varid_obs_sstp, start, count, obs_t);
    
    % read and extract salinity
    [mean_s, std_s, obs_s] = rdwoa('s', num);
    % write to netcdf file
    netcdf.putVar(nc, varid_mean_ssps, start, count, mean_s);
    netcdf.putVar(nc, varid_std_ssps, start, count, std_s);
    netcdf.putVar(nc, varid_obs_ssps, start, count, obs_s);
 
end

% close file
netcdf.close(nc);


% ----------------------------------------------------------------
% read WOA climatology file with Opendap
% ----------------------------------------------------------------
    function [mean, std, obs] = rdwoa(var, num)
  
        if strcmp(var, 't')
          var_str = 'temperature';
        else
          var_str = 'salinity';
        end
        % read decade average standard deviation (1955/2012)   
        avg_decade = 'decav';
        url = sprintf('https://data.nodc.noaa.gov/thredds/dodsC/woa/WOA13/DATAv2/%s/netcdf/%s/0.25/woa13_%s_%s%s_04v2.nc',...
        var_str, avg_decade, avg_decade, var, num);
        fprintf(1, 'read std %s: %s\n', var_str, num);
        std = ncread(url, sprintf('%c_sd', var), [1,1,2,1], [1440 720 2 1]);

        % mean and obs from the last decade 2005/2012
        avg_decade = 'A5B2';
        url = sprintf('https://data.nodc.noaa.gov/thredds/dodsC/woa/WOA13/DATAv2/%s/netcdf/%s/0.25/woa13_%s_%s%s_04v2.nc',...
        var_str, avg_decade, avg_decade, var, num);
        fprintf(1, 'read mean %s:%s\n', var_str, num);
        mean = ncread(url, sprintf('%c_an', var), [1,1,2,1], [1440 720 2 1]);
        fprintf(1, 'read obs %s:%s\n', var_str, num);
        obs  = ncread(url, sprintf('%c_dd', var), [1,1,2,1], [1440 720 2 1]);               
    end

end
